/*=============================================================================
 * Classe Connection
 * 
 * Gestion de la connexion et lancement des services.
 * WiFi / mDNS / OTA / MQTT
 * 
 * © Jean-Luc Béchennec - 2022
 */

#ifndef __CONNECTION_H__
#define __CONNECTION_H__

#include "Debug.h"
#include <stdint.h>
#include <ArduinoOTA.h>
#include <PubSubClient.h>

class Connection {
public:
  typedef enum { OFFLINE, WIFI_STBY, WIFI_OK, MDNS_OK, OTA_OK, MQTT_OK } State;

private:
  static State sState;
  static uint32_t sLastDate;
  static uint32_t sPeriod;
  static char *sSsid;
  static char *sPass;
  static char *sName;
  static char *sOTAPass;
  static char *sBrokerName;
  static IPAddress sBrokerIP;
  static WiFiClient sNet;
  static PubSubClient sClient;

  typedef void (*MessageHandlingFunction)(char *, byte *, unsigned int);
  static MessageHandlingFunction sMessageHandler; 

  typedef void (*SubscriptionFunction)();
  static SubscriptionFunction sDoSubscriptions;

  Connection() {}
  
  static void update();
  static void initOTA();
  static void incomingMessage(char *inTopic, byte *inPayload, unsigned int inLength);
  static void performSubscriptions();

#ifdef DEBUG
  static void startingOTA();
  static void OTAInProgress(unsigned int inProgress, unsigned int inTotal);
  static void endingOTA();
  static void OTAError(ota_error_t error);
#endif
  

public:
  static void setup(const char *inSsid,
                    const char *inPass,
                    const char *inName,
                    const char *inBrokerName,
                    const char *inOTAPass = NULL,
                    const uint32_t inPeriod = 1000ul);
  static void loop();
  static bool isOnline();
  static void subscribe(const char *inTopic);
  static void publish(const char *inTopic, const float inValue);
  static void publish(const char *inTopic, const double inValue);
  static void publish(const char *inTopic, const uint32_t inValue);
  static void publish(const char *inTopic, const int32_t inValue);
  static void publish(const char *inTopic, const char *inValue);
  static void publish(const char *inTopic, String &inValue);
  static void attach(SubscriptionFunction inSubFunction, MessageHandlingFunction inMessageHandler);
};

#endif /* __CONNECTION_H__ */
